//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import <UIKit/UIKit.h>
#import "PCShareResource+GridCell.h"
#import "PCShareResource+Info.h"
#import "ImageDownloadView.h"
#import "PCShareResourceDownloader.h"

@implementation PCShareResource (GridCell)

// Below method set default pic to imageView , when contentToken is unavailable for this file.
- (UIImage *)defaultPic {
    if ([[self type] isEqualToString:@"file"]) {
        PCShareResource *sharedResource = (PCShareResource *)self;
        UIImage *defaultImage = [UIImage imageNamed:@"DefaultPic"];
        
        if (sharedResource.fileType == imageSharedFileType) {
            defaultImage = [UIImage imageNamed:@"DefaultPic"];
        } else if (sharedResource.fileType  == videoSharedFileType) {
            defaultImage = [UIImage imageNamed:@"VideoDefault"];
        } else if (sharedResource.fileType  == musicSharedFileType) {
            defaultImage = [UIImage imageNamed:@"MusicDefault"];
        }
        return defaultImage;
        
    } else if([[self type] isEqualToString:@"playlist"]) {
        return [UIImage imageNamed:@"MusicDefault"];
    }else{
        return [UIImage imageNamed:@"Folder"];
    }
}

// Below method set thumbnail image to an image view. This method checks for cached image, if not found it downloads image using the content token stored in PCShareResource. If this fails a default image is set for the image view.
- (void)setThumbnailForImageView:(ImageDownloadView *)imageView withCompletionHandler:(void (^)(BOOL didSetImage))completion {
    if ([[self type] isEqualToString:@"file"]) {
        PCShareResource *resource = (PCShareResource *)self;
        imageView.imageID = resource.contentToken; // imageID needs to be set to encounter cell reusability feature
        UIImage *thumbnailImage = [UIImage imageWithContentsOfFile:[self thumbnailCachePath]];
        
        if (thumbnailImage) {
            imageView.image = thumbnailImage;
            completion(YES);
        } else {
            imageView.image = nil;
            // Intialize the PCThumbnailAPIParams object.
            PCThumbnailAPIParams *params = [[PCThumbnailAPIParams alloc] init];
            params.th = 320;
            params.tw = 320;
            params.contentToken = resource.contentToken;
            
            __weak PCShareResource *blocksafeSelf = self;
            /*
             @Description :
             This API is used to download thumbnail image of resource. Once the download process is succeed/failed result will come with completion block.
             
             @Parameters :
             PCThumbnailAPIParams:
             * This class stores all the parameters required to download thumbnail image of resource. It contains attributes such as contentToken, size, th and tw.
             * For providing access token to SDK user have to logged In.
             * contentToken and Either 'size' or ('th' and 'tw') is required parameter to call this API.
             * value for size are (Thumbnail size: 'xs' (24x24), 's' (64x64), or 'm' (128x128).
             * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
             * More information about parameters you will find in PCThumbnailAPIParams class.
             
             @CompletionHandler :
             This completion block gets called once the download process is succeed/failed. completion block contains below parameter.
             PCAPIQueryResponse : Stores all the server response and error for this API. dowloaded result comes in downloadedURL attribute.
             
             @To download thumbnail image of resource, you must take the following steps:
             * Include VerizonCloudAPI.framework header file in your class.
             #import <VerizonCloudAPI/VerizonCloudAPI.h>
             * Initalize the PCThumbnailAPIParams class and provide the required parameter.
             * Initalize the PCAPISDK object and call the downloadThumbnail:withCompletionHandler: API.
             * Perform the downloadThumbnail:withCompletionHandler: API call using the code sample below.
             * Get the result (succeed or failed) in completion block.
             * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
             */
            PCAPI *thumbnailAPI = [[PCAPISDK sharedInstance] downloadThumbnail:params withCompletionHandler:^(PCAPIQueryResponse *response) {
                // Get the result in this block.
                NSData *fileData = [NSData dataWithContentsOfURL:response.downloadedURL];
                
                if (fileData) {
                    [fileData writeToFile:[blocksafeSelf thumbnailCachePath] atomically:YES];
                }
                
                if ([imageView.imageID isEqualToString:blocksafeSelf.contentToken]) { // cell is still representing same resource
                    if (fileData) {
                        UIImage *thumbImage = [UIImage imageWithContentsOfFile:[blocksafeSelf thumbnailCachePath]];
                        
                        if (thumbImage) {
                            imageView.image = thumbImage;
                            completion(YES);
                        } else {
                            imageView.image = [blocksafeSelf defaultPic];
                            completion(YES);
                        }
                    } else {
                        imageView.image = [blocksafeSelf defaultPic];
                        completion(YES);
                    }
                } else {
                    completion(NO);
                }
            }];
            imageView.api = thumbnailAPI;
        }
    } else {
        imageView.imageID = nil;
        imageView.image = [self defaultPic];
        completion(YES);
    }
}

// Below method set a dowloaded shared images to an image view. This method checks for cached image, if not found it downloads image using the resource url stored in PCShareResource. If this fails a default image is set for the image view.
- (void)setFullResolutionImageView:(ImageDownloadView *)imageView withProgressHandler:(void (^)(float progress))progress andCompletionHandler:(void (^)(BOOL didSetImage))completion {
    if (self.fileType == imageSharedFileType) {
        PCShareResource *resource = (PCShareResource *)self;
        imageView.imageID = resource.contentToken; // imageID needs to be set to encounter cell reusability feature
        
        UIImage *image = [[UIImage alloc] initWithContentsOfFile:[self fileCachePath]];
        
        if (image) {
            imageView.image = image;
            completion(YES);
        } else {
            imageView.image = nil;
            
        __weak PCShareResource *blocksafeSelf = self;
        PCShareResourceDownloader *api = [[PCShareResourceDownloader alloc] init];
        [api downloadSharedResource:resource.url progress:^(long long bytesWritten, long long totalBytesWritten, long long totalBytesExpectedToWrite) {
                // Get the Progress in this block.
            dispatch_async(dispatch_get_main_queue(), ^(void) {
                progress((float)totalBytesWritten/[[self size] floatValue]);
            });
        } withCompletionHandler:^(NSURLResponse *response, NSURL *location, NSError *error) {
            if (!error) {
                dispatch_async(dispatch_get_main_queue(), ^(void) {
                    NSData *fileData = [NSData dataWithContentsOfURL:location];
                        
                        if (fileData) {
                            [fileData writeToFile:[self fileCachePath] atomically:YES];
                        }
                        
                        if ([imageView.imageID isEqualToString:blocksafeSelf.contentToken]) { // cell is still representing same resource
                            if (fileData) {
                                UIImage *fullResImage = [UIImage imageWithContentsOfFile:[blocksafeSelf fileCachePath]];
                                if (fullResImage) {
                                    imageView.image = fullResImage;
                                    completion(YES);
                                } else {
                                    imageView.image = [blocksafeSelf defaultPic];
                                    completion(YES);
                                }
                            } else {
                                imageView.image = [blocksafeSelf defaultPic];
                                completion(YES);
                            }
                        } else {
                            completion(NO);
                        }
                    });
                }
            }];
             imageView.api = api;
        }
    } else {
        imageView.imageID = nil;
        imageView.image = [self defaultPic];
        completion(YES);
    }
}

@end
